#!/usr/bin/env python

import sys, random, time
import urllib
import simplejson
from base64 import b64encode, b64decode

import hmac
from hashlib import sha256
from pycryptopp.cipher.aes import AES
from jpake import JPAKE

def wait_for_msg(num, channel_url):
    while True:
        msgs_j = urllib.urlopen(channel_url).read()
        msg_tuples = [simplejson.loads(m) for m in simplejson.loads(msgs_j)]
        msgs = dict(msg_tuples)
        if num in msgs:
            return msgs[num]
        time.sleep(1)

def send_msg(channel_url, num, message):
    # message is a JSONable object
    M = simplejson.dumps([num, message])
    f = urllib.urlopen(channel_url, M)
    if f.getcode() != 200:
        raise RuntimeError("HTTP response code was %d" % f.getcode())
    response = f.read()

def create_channel(baseurl, name):
    f = urllib.urlopen(baseurl+"create_channel",
                       urllib.urlencode({"name": channel}))
    if f.getcode() != 200:
        raise RuntimeError("HTTP response code was %d" % f.getcode())
    response = f.read()

def delete_channel(baseurl, name):
    f = urllib.urlopen(baseurl+"delete_channel",
                       urllib.urlencode({"name": channel}))
    if f.getcode() != 200:
        raise RuntimeError("HTTP response code was %d" % f.getcode())
    response = f.read()

# A->B: M1 = (1,1A)
# B->A: M2 = (2,(1B,2B))
# A->B: M3 = (3,(2A,hhkey,Ecreds,MACcreds))
# B->A: M4 = (4,hkey)

baseurl = "http://localhost:9001/"

words = """adorable beautiful charming dainty delightful pleasant pretty
bobcat cheetah cougar jaguar kitten kitty leopard lion lynx ocelot panther
puma tabby tiger tom tomcat
""".strip().split()

def do_sender(channel, creds):
    print "I start with the credentials: %s" % creds
    print "initiating transfer.."
    print "creating channel.."
    create_channel(baseurl, channel)
    channel_url = baseurl+"channel_"+channel
    secret = random.choice(words)
    print "=====  The secret word is:", secret
    sys.stdout.flush()
    j = JPAKE(secret)
    m1a = j.one()
    print "posting M1.."
    send_msg(channel_url, 1, m1a)
    m1b,m2b = wait_for_msg(2, channel_url)
    m2a = j.two(m1b)
    shared = j.three(m2b)
    print "guess claimed!"
    hkey = sha256(shared).digest()
    hhkey = sha256(hkey).digest()
    enc_creds = AES(shared).process(creds)
    hmac_creds = hmac.HMAC(shared, creds, sha256).digest()
    M3 = [m2a, b64encode(hhkey), b64encode(enc_creds), b64encode(hmac_creds)]
    print "posting M3.."
    send_msg(channel_url, 3, M3)
    print "waiting for M4.."
    M4 = wait_for_msg(4, channel_url)
    their_hkey = b64decode(M4)
    if their_hkey == hkey:
        print "credentials sent successfully (to somebody)!"
        print "TRANSFER SUCCESSFUL"
    else:
        print "somebody guessed wrong"
        print "TRANSFER FAILED"
    delete_channel(baseurl, channel)
    

def do_receiver(channel):
    print "I'm ready to receive the credentials"
    secret = raw_input("what is the secret word?: ")
    print "secret '%s'" % secret
    print "great! starting the transfer.."
    channel_url = baseurl+"channel_"+channel
    j = JPAKE(secret)
    m1b = j.one()
    m1a = M1 = wait_for_msg(1, channel_url)
    print "got M1"
    m2b = j.two(m1a)
    print "posting M2.."
    send_msg(channel_url, 2, [m1b,m2b])
    print "waiting for M3.."
    M3 = wait_for_msg(3, channel_url)
    m2a, hhkey_b64, enc_creds_b64, hmac_creds_b64 = M3
    shared = j.three(m2a)
    hkey = sha256(shared).digest()
    hhkey = sha256(hkey).digest()
    # let the sender know we've finished the protocol
    send_msg(channel_url, 4, b64encode(hkey))
    their_hhkey = b64decode(hhkey_b64)
    if their_hhkey != hhkey:
        print ".. except they guessed wrong"
        print "TRANSFER FAILED"
        return
    creds = AES(shared).process(b64decode(enc_creds_b64))
    hmac_creds = hmac.HMAC(shared, creds, sha256).digest()
    if hmac_creds != b64decode(hmac_creds_b64):
        print ".. encrypted credentials failed HMAC: corruption"
        print "TRANSFER FAILED"
        return
    print "TRANSFER SUCCESSFUL"
    print "the creds are", creds
    return

channel = sys.argv[1]
if len(sys.argv)>2:
    do_sender(channel, sys.argv[2])
else:
    do_receiver(channel)
