#!/usr/bin/twistd -noy server.py

import simplejson
import urllib
from twisted.application import service, internet
from twisted.web import server, static
from twisted.web.resource import Resource

class Channel(Resource):
    isLeaf = True
    def __init__(self):
        Resource.__init__(self)
        self.messages = []
    def render_GET(self, req):
        out = simplejson.dumps(self.messages)
        return out
    def render_POST(self, req):
        req.content.seek(0)
        message = req.content.read()
        self.messages.append(message)
        return "Message added to queue (len=%d).\n" % len(message)

class ChannelAdder(Resource):
    def __init__(self, parent):
        Resource.__init__(self)
        self.parent = parent
    def render_POST(self, req):
        name = req.args["name"][0]
        assert urllib.quote(name) == name
        self.parent.putChild("channel_"+name, Channel())
        return "Channel %s created.\n" % name

class ChannelDeleter(Resource):
    def __init__(self, parent):
        Resource.__init__(self)
        self.parent = parent
    def render_POST(self, req):
        name = req.args["name"][0]
        assert urllib.quote(name) == name
        self.parent.delEntity("channel_"+name)
        return "Channel %s deleted.\n" % name

class Root(Resource):
    def __init__(self):
        Resource.__init__(self)
        self.putChild("x", Channel())
        self.putChild("create_channel", ChannelAdder(self))
        self.putChild("delete_channel", ChannelDeleter(self))

root = Root()

s = internet.TCPServer(9001, server.Site(root))

application = service.Application("xfer-server")
s.setServiceParent(application)
